// I2C Master handler
#include <p18cxxx.h>
#include <i2c.h>
#include "i2c_master.h"

#define SI2C_SDA PORTCbits.RC4
#define SI2C_SCL PORTCbits.RC3
#define SI2C_SDA_LAT LATCbits.LATC4
#define SI2C_SCL_LAT LATCbits.LATC3
#define SI2C_SDA_TRIS TRISCbits.TRISC4
#define SI2C_SCL_TRIS TRISCbits.TRISC3
#define SI2C_TMG 5

// I2C master interrupts
unsigned char I2CPhase,	// phase code
              I2CDevAddr,	// I2C device address
              I2CDataByte[I2C_M_BUFLEN],	// I2C data byte, max 20 bytes
              I2CLengthT,	// data byte length for transmit
              I2CLengthR,	// data byte length for receive
              I2CCount,		// data counter(pointer)
              I2CError;		// I2C error flag
unsigned char I2CDiscard;	// buffer for discard

void
si2c_wait( void )
{
  int i;

  for( i = 0; i < SI2C_TMG; i ++ ) ;
}

void 
iniz_i2c( void )
{
  SI2C_SDA = 1;		// each lines are HIGH
  SI2C_SCL = 1;
  SI2C_SDA_TRIS = 1;
  SI2C_SCL_TRIS = 1;
  I2CError = 0;
}

int
i2c_tx_byte( unsigned char d )
{
  int i, ack;

  for( i = 0; i < 8; i ++ ) {
    if( 0x80 & d ) SI2C_SDA_TRIS = 1;	// data is 1
    else           SI2C_SDA_TRIS = 0;	// data is 0
    si2c_wait();
    SI2C_SCL_TRIS = 1;
    si2c_wait();
    SI2C_SCL_TRIS = 0;
    d <<= 1;
  }
  SI2C_SDA_TRIS = 1;	// ACK receive
  si2c_wait();
  SI2C_SCL_TRIS = 1;
  si2c_wait();
  ack = SI2C_SDA;
  SI2C_SCL_TRIS = 0;
  return ack;  
}

void
i2c_stop( void )
{
  SI2C_SDA_TRIS = 0;
  si2c_wait();
  SI2C_SCL_LAT = 1;
  si2c_wait();
  SI2C_SDA_TRIS = 1;
}

void 
i2cstart( void )
{
  int i, a;
  // check for start condition
  SI2C_SDA_TRIS = 1;
  SI2C_SCL_LAT = 1;
  SI2C_SCL_TRIS = 1;
  if( 0 == SI2C_SDA || 0 == SI2C_SCL ) {
    I2CError = 1;		// bus in use
    i2c_stop();
    return; 
  }
  // start condition
  SI2C_SDA_LAT = 0;
  SI2C_SCL_LAT = 0;
  SI2C_SDA_TRIS = 0;
  si2c_wait();
  SI2C_SCL_TRIS = 0;
  si2c_wait();
  // address byte
  a = i2c_tx_byte( I2CDevAddr );
  if( 0 != a ) {
    I2CError = 1; // NACK
    i2c_stop();
    return;
  }
  for( i = 0; i < I2CLengthT; i ++ ) {
    a = i2c_tx_byte( I2CDataByte[i] );
    if( 0 != a ) {
      I2CError = 1; // NACK
      i2c_stop();
      return;
    }
  }
  I2CError = 0;
  i2c_stop();
  return;
}

void 
i2cready( void )
{
}
